config = {
    "video_quality": None,
    "danmu_mode": None,
    "video_stored_path": None,
    "verbose": None,
    "zhubo_room_url": None,  # 命令行获取的配置
}

room_status = {
    "sd_rmtp_url": None,
    "hd_rmtp_url": None,
    "spd_rmtp_url": None,
    "id": None,
    "name": None,
    "gg_show": None,
    "owner_uid": None,
    "owner_name": None,
    "room_url": None,
    "near_show_time": None,
    "username": None,
    "tags": None,
    "live_stat": None,
    "fans_count": None,
    "weight": None,
    "is_finished": False  # 当标记为 True的时候,这个时候所有的
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
对mplayer及其他播放器(TODO)的控制

    player = MPlayer()

方法:
    player.start(url)
    player.pause()
    player.quit()
    player.loop()
    player.set_volume(50)
    player.time_pos
    player.is_alive

queue自定义get_song方法, 从中取出url, 进行播放(暂时, 以后可以抽象)
    player.start_queue(queue)

如果需要更新,更换播放列表直接重复上面命令即可
    player.start_queue(queue)
"""
import subprocess
import logging
import signal
# import fcntl
import time
import abc
import os
from threading import Thread, Event

logger = logging.getLogger('danmu.fm.player')


class NotPlayingError(Exception):
    """对播放器操作但播放器未在运行"""
    pass


class PlayerUnavailableError(Exception):
    """该播放器在该系统上不存在"""
    pass


class Player(object):
    """所有播放器的抽象类"""

    __metaclass__ = abc.ABCMeta

    # Command line name for the player (e.g. "mplayer")
    _player_command = ""

    # Default arguments (excluding command name)
    _default_args = []

    _null_file = open(os.devnull, "w")

    @abc.abstractmethod
    def __init__(self, default_volume=50):
        """初始化

        子类需要先判断该播放器是否可用（不可用则抛出异常），再调用该方法
        event: 传入的一个 Event ，用于通知播放完成
        default_volume: 默认音量
        """
        self.sub_proc = None            # subprocess instance
        self._args = [self._player_command] + self._default_args
        self._exit_event = Event()
        self._volume = default_volume

    def __repr__(self):
        if self.is_alive:
            status = 'PID {0}'.format(self.sub_proc.pid)
        else:
            status = 'not running'
        return '<{0} ({1})>'.format(self.__class__.__name__, status)

    def _run_player(self, extra_cmd):
        """
        运行播放器（若当前已有正在运行的，强制推出）

        extra_cmd: 额外的参数 (list)
        """
        # Force quit old process
        if self.is_alive:
            self.quit()
        args = self._args + extra_cmd
        logger.debug("Exec: " + ' '.join(args))
        self.sub_proc = subprocess.Popen(
            args,
            stdin=self._null_file,
            stdout=self._null_file,
            stderr=self._null_file,
            preexec_fn=os.setsid
        )
        # Set up NONBLOCKING flag for the pipe
        # flags = fcntl.fcntl(self.sub_proc.stdout, fcntl.F_GETFL)
        # flags |= os.O_NONBLOCK
        # fcntl.fcntl(self.sub_proc.stdout, fcntl.F_SETFL, flags)
        # Start watchdog
        Thread(target=self._watchdog).start()

    def _watchdog(self):
        """
        监控正在运行的播放器（独立线程）

        播放器退出后将会设置 _exit_event
        """
        if not self.is_alive:
            logger.debug("Player has already terminated.")
            self._exit_event.set()
            return
        logger.debug("Watching %s[%d]",
                     self._player_command, self.sub_proc.pid)
        returncode = self.sub_proc.wait()
        logger.debug("%s[%d] exit with code %d",
                     self._player_command, self.sub_proc.pid, returncode)
        self._exit_event.set()

    @property
    def is_alive(self):
        """判断播放器是否正在运行"""
        if self.sub_proc is None:
            return False
        return self.sub_proc.poll() is None

    def quit(self):
        """退出播放器

        子类应当覆盖这个方法（但不强制），先尝试 gracefully exit ，再调用 super().quit()
        """
        if not self.is_alive:
            return
        self.sub_proc.terminate()

    # Abstract methods

    @abc.abstractmethod
    def start(self, url):
        """开始播放

        url: 歌曲地址
        """
        pass

    @abc.abstractmethod
    def pause(self):
        """暂停播放"""
        pass

    @abc.abstractmethod
    def set_volume(self, volume):
        """设置音量

        volume: 音量 (int)"""
        self._volume = volume   # int

    @abc.abstractproperty
    def time_pos(self):
        """获取当前播放时间

        返回播放时间的秒数 (int)"""
        pass


class MPlayer(Player):

    _player_command = "mplayer"
    _default_args = [
        '-slave',
        '-nolirc',          # Get rid of a warning
        '-quiet',           # Cannot use really-quiet because of get_* queries
        '-softvol',         # Avoid using hardware (global) volume
        '-cache', '1024',   # Use 5MiB cache
        '-cache-min', '0.1'   # Start playing after 2% cache filled
    ]

    def __init__(self, *args):
        super(MPlayer, self).__init__(*args)
        self._exit_queue_event = False
        self._loop = False
        self._pause = False
        self._time = 0

    def _watchdog_queue(self):
        self._exit_queue_event = True

        while self._exit_queue_event:
            if self._loop:
                self.start(self.queue.get_playingsong()['url'])
            else:
                self.start(self.queue.get_song()['url'])
            self.sub_proc.wait()  # Wait for event

    def start_queue(self, queue, volume=None):
        self.queue = queue
        self._volume = volume if volume else self._volume

        if not self._exit_queue_event:
            Thread(target=self._watchdog_queue).start()
        else:
            try:
                self.sub_proc.terminate()
            except OSError:
                logger.info('wrong with start_queue')

    def loop(self):
        self._loop = False if self._loop else True

    def next(self):
        self.start_queue(self.queue, self._volume)

    def start(self, url):
        self._run_player(['-volume', str(self._volume), url])

    def pause(self):
        """
        pasue状态下如果取时间会使歌曲继续, 这里加了一个_pause状态
        """
        self._pause = False if self._pause else True
        self._send_command('pause')

    def quit(self):
        # Force quit the whole process group of mplayer.
        # mplayer will not respond during network startup
        # and has two processes in slave mode.
        self._exit_queue_event = False  # 标记_watchdog_queue结束
        if not self.is_alive:
            return
        try:
            os.killpg(os.getpgid(self.sub_proc.pid), signal.SIGKILL)
        except OSError:
            # The player might have exited itself.
            pass

    @property
    def time_pos(self):

        try:
            if self._pause:  # 暂停时直接返回_time
                return self._time
            songtime = self._send_command('get_time_pos', 'ANS_TIME_POSITION')
            if songtime:
                self._time = int(round(float(songtime)))
                return self._time
            else:
                return 0
        except NotPlayingError:
            return 0

    def set_volume(self, volume):
        # volume <value> [abs] set if abs is not zero, otherwise just add delta
        self._volume = volume
        self._send_command("volume %d 1" % volume)
        super(MPlayer, self).set_volume(volume)

    # Special functions for mplayer

    def _send_command(self, cmd, expect=None):
        """Send a command to MPlayer.

        cmd: the command string
        expect: expect the output starts with a certain string
        The result, if any, is returned as a string.
        """
        if not self.is_alive:
            raise NotPlayingError()
        logger.debug("Send command to mplayer: " + cmd)
        cmd = cmd + "\n"
        # In Py3k, TypeErrors will be raised because cmd is a string but stdin
        # expects bytes. In Python 2.x on the other hand, UnicodeEncodeErrors
        # will be raised if cmd is unicode. In both cases, encoding the string
        # will fix the problem.
        try:
            self.sub_proc.stdin.write(cmd)
        except (TypeError, UnicodeEncodeError):
            self.sub_proc.stdin.write(cmd.encode('utf-8', 'ignore'))
        time.sleep(0.1)     # wait for mplayer (better idea?)
        # Expect a response for 'get_property' only
        if not expect:
            return
        while True:
            try:
                output = self.sub_proc.stdout.readline().rstrip()
            except IOError:
                return None
            # print output
            split_output = output.split('=')
            # print(split_output)
            if len(split_output) == 2 and split_output[0].strip() == expect:
                # We found it
                value = split_output[1]
                return value.strip()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
弹幕fm主程序
"""
import argparse
import gettext
import io
import json
import logging
import os
import re
import sys
import time
from .settings import ROOT_PROJECT_DIR
from .cmd_config import config
# from threading import Thread
# import subprocess
from .client.douyu_client import DouyuClient
import traceback

APP_DESC = """
    _____                                ______ __  __
    |  __ \                              |  ____|  \/  |
    | |  | | __ _ _ __  _ __ ___  _   _  | |__  | \  / |
    | |  | |/ _` | '_ \| '_ ` _ \| | | | |  __| | |\/| |
    | |__| | (_| | | | | | | | | | |_| |_| |    | |  | |
    |_____/ \__,_|_| |_|_| |_| |_|\__,_(_)_|    |_|  |_|

                        ---- A Terminal Tools For DouyuTV

    @author Micheal Gardner (twocucao@gmail.com)
                                    last_update 2017-07-10 08:54:59
"""

logger = logging.getLogger('danmu.fm')


def check_setting_and_env():
    logger.info("程序正在启动,检查环境配置")
    if sys.version_info < (3, 2):
        raise RuntimeError("at least Python 3.3 is required!!")
    # Session = scoped_session(sessionmaker(bind=ENGINE))
    # s = Session()
    # try:
    #     rs_version = s.execute("SELECT VERSION();")
    #     version = rs_version.fetchone()[0]
    #     env_infomation = """\
    #     ------------------------------------------------------------
    #     PostgreSQL 版本信息 : {0}
    #     ------------------------------------------------------------
    #     """.format(version)
    #     print(env_infomation)
    #     Session.remove()
    # except sqlalchemy.exc.OperationalError as e:
    #     logger.error("没有初始化数据库")
    #     # 貌似没什么作用?
    #     try:
    #         conn = ENGINE.connect()
    #         conn.execute("commit")
    #         conn.execute("create database danmufm")
    #         conn.close()
    #     except sqlalchemy.exc.OperationalError as e:
    #         logger.error("无法创建数据库,请检查配置")
    #         logger.info("程序退出")
    #         exit()
    #     pass

    logger.info("开始配置环境")


import click


@click.command()
@click.argument('url', required=True)
@click.option('-q', '--quality', default=0, help='查看视频清晰度, 0: 无, 1:流畅, 2:普通, 3:高清,')
@click.option('-m', '--mode', default=0, help='选择弹幕类型,0为默认普通弹幕,1为海量弹幕')
@click.option('-p', '--path', default=".", type=click.Path(), help='视频缓存本地地址,注:quality必须要为1-2-3其中之一')
@click.option('-t', '--thread', default=10, help='选择弹幕获取线程数,默认为10,线程限制在2~50之间')
@click.option('-v', '--verbose', count=True, help='-v 为普通日志模式, -vvvv 超级海量日志模式')
def parse_command(quality, mode, path, thread, verbose, url):
    """
指定获取主播的房间地址对主播直播情况进行抓取与统计 (Mac and Ubuntu Only)

Example:

    danmu.fm -q 2 -v 1 http://www.douyu.com/qiuri

    danmu.fm -q 3 -m 0 -p "videos/20160609_1900_2240_小苍.mp4" -v 1 http://www.douyu.com/qiuri

    """

    # danmu.fm -q 3 -m 0 -p "videos/20160609_1900_2240_小苍.mp4"-v 1 http://www.douyu.com/qiuri
    config["video_quality"] = quality if quality > -1 or quality < 4 else 0
    config["danmu_mode"] = mode if mode > -1 and mode < 2 else 0
    current_working_dir = os.getcwd()
    config["video_stored_path"] = os.path.join(current_working_dir,path) if path != "." else current_working_dir
    config["thread_num"] = thread if thread >= 2 or mode <= 50 else 10
    config["verbose"] = verbose
    # [TODO:如果等于1就普通,如果为4直接开启全局日志]
    config["zhubo_room_url"] = url
    logger.info("正在检查环境")
    check_setting_and_env()
    logger.info("环境检查完毕,正在开启斗鱼客户端(请等待15s~30s)")
    start_douyu_client()


def start_douyu_client():
    url = config["zhubo_room_url"]
    DouyuClient(url).start()


def main():
    print(APP_DESC)
    parse_command()


if __name__ == "__main__":
    main()



import logging
import curses


class ColorStreamHandler(logging.Handler):
    def __init__(self, use_colors):
        logging.Handler.__init__(self)
        self.use_colors = use_colors

        # Initialize environment
        curses.setupterm()

        # Get the foreground color attribute for this environment
        self.fcap = curses.tigetstr('setaf')

        # Get the normal attribute
        self.COLOR_NORMAL = curses.tigetstr('sgr0').decode("utf-8")

        # Get + Save the color sequences
        self.COLOR_INFO = curses.tparm(self.fcap, curses.COLOR_GREEN).decode("utf-8")
        self.COLOR_ERROR = curses.tparm(self.fcap, curses.COLOR_RED).decode("utf-8")
        self.COLOR_WARNING = curses.tparm(self.fcap, curses.COLOR_YELLOW).decode("utf-8")
        self.COLOR_DEBUG = curses.tparm(self.fcap, curses.COLOR_BLUE).decode("utf-8")

    def color(self, msg, level):
        if level == "INFO":
            return "%s%s%s" % (self.COLOR_INFO, msg, self.COLOR_NORMAL)
        elif level == "WARNING":
            return "%s%s%s" % (self.COLOR_WARNING, msg, self.COLOR_NORMAL)
        elif level == "ERROR":
            return "%s%s%s" % (self.COLOR_ERROR, msg, self.COLOR_NORMAL)
        elif level == "DEBUG":
            return "%s%s%s" % (self.COLOR_DEBUG, msg, self.COLOR_NORMAL)
        else:
            return msg

    def emit(self, record):
        # record.msg = record.msg.encode('utf-8', 'ignore')
        msg = self.format(record)

        # This just removes the date and milliseconds from asctime
        temp = msg.split(']')
        msg = '[' + temp[0].split(' ')[1].split(',')[0] + ']' + temp[1]
        if self.use_colors:
            msg = self.color(msg, record.levelname)
        print(msg)



import asyncore
import datetime
import re
import uuid
import hashlib
import socket
import requests
import json
import threading
import time
from ..misc.player import MPlayer
from ..misc.color_printer import ColorPrinter
from ..model.douyu_msg import DouyuMsg
from ..cmd_config import config, room_status
from queue import Queue
# import sys
import logging
from concurrent.futures import ThreadPoolExecutor

logger = logging.getLogger('danmu.fm')
fetched_msg_q = Queue()


class DouyuDanmuManager(object):
    """貌似获取弹幕到了上限了?当55开500W的时候,获取量在每秒8条左右,这个时候线程明显是不够用的.正在使用异步代替"""

    def __init__(self, auth_dst_ip, auth_dst_port):
        self.DANMU_ADDR = ("danmu.douyutv.com", 8602)
        self.DANMU_AUTH_ADDR = (auth_dst_ip, int(auth_dst_port))
        self.room_id = str(room_status["id"])
        self.auth_dst_ip = auth_dst_ip
        self.auth_dst_port = auth_dst_port
        self.dev_id = str(uuid.uuid4()).replace("-", "")

    def start(self):
        self.do_login()
        if self.live_stat == "离线":
            logger.info("主播离线中,正在退出...")
        else:
            logger.info("主播在线中,准备获取弹幕...")
            t = threading.Thread(target=self.keeplive)
            t.setDaemon(True)
            t.start()
            MAX_THREAD_WORKS = config.get("thread_num",10)
            pool = ThreadPoolExecutor(MAX_THREAD_WORKS)
            pipeline = pool.submit(self.consume_fetched_msg,fetched_msg_q)
            for i in range(MAX_THREAD_WORKS):
                pool.submit(self.keep_fetch_msg)
            pipeline.result()

    def keep_fetch_msg(self):
        while True:
            self.get_danmu()

    def consume_fetched_msg(self,in_q):
        while True:
            item = in_q.get()
            self.parse_recv_msg(item)
        pass
    def keeplive(self):
        logger.info("启动 KeepLive 线程")
        while True:
            self.send_auth_keeplive_msg()
            self.send_danmu_keeplive_msg()
            time.sleep(40)

    def do_login(self):
        self.danmu_auth_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.danmu_auth_socket.connect(self.DANMU_AUTH_ADDR)
        self.danmu_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.danmu_socket.connect(self.DANMU_ADDR)
        self.send_auth_loginreq_msg()
        recv_msg = self.auth_recv()
        logger.debug(recv_msg)
        if "live_stat@=0" in recv_msg:
            self.live_stat = "离线"
            room_status["live_stat"] = self.live_stat
        else:
            self.live_stat = "在线"
            room_status["live_stat"] = self.live_stat
            self.username = re.search('\/username@=(.+)\/nickname', recv_msg).group(1)
            recv_msg = self.auth_recv()
            self.gid = re.search('\/gid@=(\d+)\/', recv_msg).group(1) if config["danmu_mode"] == 0 else '-9999'# 取消注释可以直接使用正常方式
            self.weight = re.search('\/weight@=(\d+)\/', recv_msg).group(1)
            self.fans_count = re.search('\/fans_count@=(\d+)\/', recv_msg).group(1)
            self.send_qrl_msg()
            recv_msg = self.auth_recv()
            logger.debug(recv_msg)
            # recv_msg = self.auth_recv()
            # print(recv_msg)
            self.send_auth_keeplive_msg()
            recv_msg = self.auth_recv()
            logger.debug(recv_msg)
            self.send_danmu_loginreq_msg()
            recv_msg = self.danmu_recv()
            logger.debug(recv_msg)
            self.send_danmu_join_group_msg()

    def parse_recv_msg(self,recv_msg):
        if "type@=" not in recv_msg:
            logger.debug(recv_msg)
        elif "type@=error" in recv_msg:
            logger.error("错误消息,可能认证失效")
        else:
            msg_content = recv_msg.replace("@S", "/").replace("@A=", ":").replace("@=", ":")
            # print(msg_content)
            try:
                msg_type = re.search('type:(.+?)\/', msg_content).group(1)

                if msg_type == "chatmsg":
                    msg_type_zh = "弹幕消息"
                    sender_id = re.search('\/uid:(.+?)\/', msg_content).group(1) if 'uid:' in msg_content else "unknown"
                    nickname = re.search('\/nn:(.+?)\/', msg_content).group(1) if 'nn:' in msg_content else "unknown"
                    content = re.search('\/txt:(.+?)\/', msg_content).group(1) if 'txt:' in msg_content else "unknown"
                    level = re.search('\/level:(.+?)\/', msg_content).group(1) if 'level:' in msg_content else "unknown"
                    client_type = re.search('\/ct:(.+?)\/', msg_content).group(
                        1) if 'ct:' in msg_content else "unknown"  # ct 默认值 0 web
                    time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                    ColorPrinter.green(
                        "|" + msg_type_zh + "| " + self.align_left_str(nickname, 20, " ") + self.align_left_str(
                            "<Lv:" + level + ">", 8, " ") + self.align_left_str("(" + sender_id + ")", 13,
                                                                                " ") + self.align_left_str(
                            "[" + client_type + "]", 10, " ") + "@ " + time + ": " + content + " ")

                elif msg_type == "uenter":
                    msg_type_zh = "入房消息"
                    user_id = re.search('\/uid:(.+?)\/', msg_content).group(1) if 'uid:' in msg_content else "unknown"
                    nickname = re.search('\/nn:(.+?)\/', msg_content).group(1) if 'nn:' in msg_content else "unknown"
                    strength = re.search('\/str:(.+?)\/', msg_content).group(1) if 'str:' in msg_content else "unknown"
                    level = re.search('\/level:(.+?)\/', msg_content).group(1) if 'level:' in msg_content else "unknown"
                    time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                    ColorPrinter.red(
                        "|" + msg_type_zh + "| " + self.align_left_str(nickname, 20, " ") + self.align_left_str(
                            "<Lv:" + level + ">", 8, " ") + self.align_left_str("(" + user_id + ")", 13,
                                                                                " ") + self.align_left_str(
                            "[" + strength + "]", 10, " ") + "@ " + time)

                elif msg_type == "dgb":
                    msg_type_zh = "礼物赠送"
                    level = re.search('\/level:(\d+?)\/', msg_content).group(
                        1) if 'level:' in msg_content else "unknown"
                    user_id = re.search('\/uid:(.+?)\/', msg_content).group(1) if 'uid:' in msg_content else "unknown"
                    nickname = re.search('\/nn:(.+?)\/', msg_content).group(1) if 'nn:' in msg_content else "unknown"
                    strength = re.search('\/str:(.+?)\/', msg_content).group(1) if 'str:' in msg_content else "unknown"
                    dw = re.search('\/dw:(.+?)\/', msg_content).group(1) if 'dw:' in msg_content else "unknown"
                    gs = re.search('\/gs:(.+?)\/', msg_content).group(1) if 'gs:' in msg_content else "unknown"
                    hits = re.search('\/hits:(.+?)\/', msg_content).group(1) if 'hits:' in msg_content else "unknown"
                    time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                    ColorPrinter.yellow(
                        "|" + msg_type_zh + "| " + self.align_left_str(nickname, 20, " ") + self.align_left_str(
                            "<Lv:" + level + ">", 8, " ") + self.align_left_str("(" + user_id + ")", 13,
                                                                                " ") + self.align_left_str(
                            "[" + dw + "]", 10, " ") + self.align_left_str("[" + gs + "]", 10,
                                                                           " ") + self.align_left_str(
                            "[" + strength + "]", 10, " ") + "@ " + time + ": " + hits + " hits ")
            except Exception as e:
                print(e)
                logger.error("弹幕内容解析错误")

    def get_danmu(self):
        recv_msg = self.danmu_recv()
        fetched_msg_q.put(recv_msg)

    def parse_content(self, msg):
        # print(msg)
        content = msg[12:-1].decode('utf-8', 'ignore')
        return content

    def danmu_recv(self):
        return self.parse_content(self.danmu_socket.recv(4000))

    def auth_recv(self):
        return self.parse_content(self.danmu_auth_socket.recv(4000))

    def send_auth_keeplive_msg(self):
        data = "type@=keeplive/tick@=" + self.timestamp() + "/vbw@=0/k@=19beba41da8ac2b4c7895a66cab81e23/"
        msg = self.message(data)
        self.danmu_auth_socket.sendall(msg)

    def send_danmu_keeplive_msg(self):
        data = "type@=keeplive/tick@=" + self.timestamp() + "/"
        msg = self.message(data)
        self.danmu_socket.sendall(msg)

    def send_danmu_join_group_msg(self):
        data = "type@=joingroup/rid@=" + self.room_id + "/gid@=" + self.gid + "/"
        msg = self.message(data)
        self.danmu_socket.sendall(msg)

    def send_qrl_msg(self):
        data = "type@=qrl/rid@=" + self.room_id + "/"
        msg = self.message(data)
        self.danmu_auth_socket.sendall(msg)

    def send_danmu_loginreq_msg(self):
        data = "type@=loginreq/username@=" + self.username + "/password@=1234567890123456/roomid@=" + self.room_id + "/"
        msg = self.message(data)
        self.danmu_socket.sendall(msg)

    def send_auth_loginreq_msg(self):
        time = self.timestamp()
        vk = hashlib.md5(bytes(time + "7oE9nPEG9xXV69phU31FYCLUagKeYtsF" + self.dev_id, 'utf-8')).hexdigest()
        data = "type@=loginreq/username@=/ct@=0/password@=/roomid@=" + self.room_id + "/devid@=" + self.dev_id + "/rt@=" + self.timestamp() + "/vk@=" + vk + "/ver@=20150929/"
        msg = self.message(data)
        self.danmu_auth_socket.sendall(msg)

    def timestamp(self):
        return str(int(time.time()))

    def message(self, content):
        return DouyuMsg(content).get_bytes()

    def align_left_str(self, raw_str, max_length, filled_chr):
        my_length = 0
        for i in range(0, len(raw_str)):
            if ord(raw_str[i]) > 127 or ord(raw_str[i]) <= 0:
                my_length += 1

            my_length += 1

        if (max_length - my_length) > 0:
            return raw_str + filled_chr * (max_length - my_length)
        else:
            return raw_str



from datetime import datetime
from sqlalchemy import Table , Column, Integer, String, ForeignKey, TEXT, DATE, Numeric, create_engine
from sqlalchemy.engine.url import URL
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref

Base = declarative_base()

class DanmuMsg(object):
    """仅仅用来存储能够获取到(并不是所有)的弹幕消息"""
    __tablename__ = "danmu_msgs"
    msg_type = Column(String(10))
    uid      = Column(String(20))
    level    = Column(String(100))
    content  = Column(Integer())
    rid      = Column(String(20))
    uid      = Column(String(20))
# chatmsg
# type@=chatmsg/rid@=301712/gid@=-9999/uid@=123456/nn@=test/txt@=666/level@=1/




class DouyuMsg(object):

    """Docstring for DouyuMsg. """

    def __init__(self,content):
        self.length = bytearray([len(content) + 9, 0x00, 0x00, 0x00])
        self.code = self.length
        self.magic = bytearray([0xb1, 0x02, 0x00, 0x00])
        self.content = bytes(content.encode("utf-8"))
        self.end = bytearray([0x00])


    def get_bytes(self):
        return bytes(self.length + self.code + self.magic + self.content + self.end)


if __name__ == "__main__":
    print(DouyuMsg("type").get_bytes())




class ColorPrinter:
    RED = '\033[91m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    LIGHT_PURPLE = '\033[94m'
    PURPLE = '\033[95m'
    END = '\033[0m'

    @classmethod
    def red(cls, s):
        print(cls.RED + s + cls.END)

    @classmethod
    def green(cls, s):
        print(cls.GREEN + s + cls.END)

    @classmethod
    def yellow(cls, s):
        print(cls.YELLOW + s + cls.END)

    @classmethod
    def lightPurple(cls, s):
        print(cls.LIGHT_PURPLE + s + cls.END)

    @classmethod
    def purple(cls, s):
        print(cls.PURPLE + s + cls.END)



from subprocess import check_output
from urllib.parse import unquote
from urllib.request import urlopen
import hashlib
import json
import logging
import os
import re
import requests
import subprocess
import threading
import time

from ..cmd_config import room_status, config
from ..settings import CURRENT_USER_HOME_DIR
from .douyu_danmu_manager import DouyuDanmuManager
from danmufm.misc.player import MPlayer

logger = logging.getLogger("danmu.fm")


def valid_json(my_json):
    """ 验证是否为 json 数据格式"""
    try:
        json_object = json.loads(my_json)
    except ValueError as e:
        print(e)
        return False
    return json_object


class DouyuClient(object):
    """
    [TODO:如下]
    1. 初始化Client相关配置(获取)
    2. 获取 房间信息 / 主播信息
    3. 使用DanmuManager获取弹幕信息
    4. 使用队列进行缓存和入库
    """

    def __init__(self, url):
        self.DOUYU_PREFIX = "http://www.douyu.com/"
        if self.DOUYU_PREFIX not in url:
            url = self.DOUYU_PREFIX + url
        self.url = url
        self.mplayer = MPlayer()


    def start(self):
        # 获取房间信息
        auth_server_ip, auth_server_port = self.fetch_room_info(self.url)
        if auth_server_ip == False or auth_server_port == True:
            exit()
        # 获取视频流信息
        # self.fetch_rtmp_info()
        # 使用弹幕Manager不断获取弹幕到队列中,并打印出来
        self.fetch_danmu(auth_server_ip,auth_server_port)
        pass

    def fetch_room_info(self, url):

        html = urlopen(url).read().decode()
        room_info_json = re.search('var\s\$ROOM\s=\s({.*});', html).group(1)
        # print(room_info_json)
        auth_server_json = re.search('\$ROOM\.args\s=\s({.*});', html).group(1)
        # print(auth_server_json)
        room_info_json_format = valid_json(room_info_json)
        auth_server_json_format = valid_json(auth_server_json)

        if room_info_json_format != False and auth_server_json_format != False:
            js = room_info_json_format
            room = room_status
            room["id"] = js["room_id"]
            room["name"] = js["room_name"]
            room["gg_show"] = js["show_id"]
            room["owner_uid"] = js["owner_uid"]
            room["owner_name"] = js["owner_name"]
            room["room_url"] = js["room_url"]
            room["near_show_time"] = js["show_time"]
            # room["tags"] = []
            # room_tags_json = js["all_tag_list"]
            # if js["room_tag_list"] != None:
            #     room_tags_size = len(js["room_tag_list"])
            #     for i in range(0, room_tags_size):
            #         room["tags"].append(room_tags_json[js["room_tag_list"][i]]["name"])

            auth_servers = valid_json(unquote(auth_server_json_format["server_config"]))
            auth_server_ip = auth_servers[0]["ip"]
            auth_server_port = auth_servers[0]["port"]
            return auth_server_ip, auth_server_port

        else:
            logger.info("请求网页错误,正在退出...")
            return False, False

    def fetch_rtmp_info(self):
        api_url_prefix = "http://douyutv.com/api/v1/"
        cctime = int(time.time())
        md5url = "room/" + str(room_status["id"]) + "?aid=android&client_sys=android&time=" + str(cctime)
        m2 = hashlib.md5(bytes(md5url + "1231", "utf-8"))
        url_json = api_url_prefix + md5url + "&auth=" + m2.hexdigest()
        res = requests.get(url_json)
        js_data = json.loads(res.text)
        print("-->" + str(js_data))
        # 如果在线,则存在RTMP视频流,否则主播不在线

        if str(js_data["data"]["rtmp_live"]).strip() == "":
            logger.error("当前主播不在线,请切换别的房间试试")
            exit()
        else:
            logger.info("当前主播在线")
        sd_rmtp_url = str(js_data["data"]["rtmp_url"]) + "/" + str(js_data["data"]["rtmp_live"])
        hd_rmtp_url = str(js_data["data"]["rtmp_url"]) + "/" + str(js_data["data"]["rtmp_live"])
        spd_rmtp_url = str(js_data["data"]["rtmp_url"]) + "/" + str(js_data["data"]["rtmp_live"])
        sd_flv_addr = requests.get(sd_rmtp_url, allow_redirects=False).headers["Location"]
        hd_flv_addr = requests.get(hd_rmtp_url, allow_redirects=False).headers["Location"]
        spd_flv_addr = requests.get(spd_rmtp_url, allow_redirects=False).headers["Location"]
        if config["video_stored_path"] != os.getcwd():
            if config["video_quality"] <= 0:
                logger.info("不播放视频")
            elif config["video_quality"] == 1:
                logger.info("正在尝试使用Mplayer播放普清视频" + sd_flv_addr)
                self.mplayer.start(sd_flv_addr)
            elif config["video_quality"] == 2:
                logger.info("正在尝试使用Mplayer播放高清视频" + hd_flv_addr)
                self.mplayer.start(hd_flv_addr)
            else:
                logger.info("正在尝试使用Mplayer播放超清视频" + spd_flv_addr)
                self.mplayer.start(spd_flv_addr)
        else:
            t = threading.Thread(target=self.wget_to_path,args=(config["video_stored_path"],spd_flv_addr,))
            t.setDaemon(True)
            t.start()
        pass
    def wget_to_path(self,path,url):
        cmd = ["/usr/local/bin/wget",
                url,
               "-O",
                os.path.join(path,room_status["owner_name"] + "_" + room_status["name"].strip().replace(" ","_") + str(time.strftime("_%Y%m%d_%H%M%S") + str(".flv")))
               ]
        logger.debug(cmd)
        try:
            check_output(cmd, stderr=subprocess.PIPE)
        except subprocess.CalledProcessError as e:
            print(e.output)

        pass





    def fetch_danmu(self, auth_server_ip, auth_server_port):
        client = DouyuDanmuManager(auth_server_ip, auth_server_port)
        client.start()

    def print_room_info(self):
        print("=========================================")
        print("= Room Infomation                       =")
        print("=========================================")
        print("= 房间: " + room_status["name"] + "(" + room_status["id"] + ")")
        print("= 主播: " + room_status["owner_name"] + str(room_status["owner_uid"]))
        print("= 公告: " + re.sub("\n+", "\n", re.sub('<[^<]+?>', '', room_status["gg_show"])))
        # print("= 标签: " + str(room_status["tags"]))
        print("= 在线: " + room_status["live_stat"])
        print("= 粉丝: " + room_status["fans_count"])
        print("= 财产: " + room_status["weight"])
        print("=========================================")



# -*- encoding: UTF-8 -*-
from setuptools import setup, find_packages

"""
打包的用的setup必须引入，
"""

VERSION = '0.3.5'

setup(name='danmu.fm',
      version=VERSION,
      package_data={'danmufm': ['template/*', ]},
      description="a tiny and smart cli player of douyu based on Python",
      long_description='just enjoy',
      classifiers=[],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='python douyu danmu danmu.fm terminal',
      author='twocucao',
      author_email='twocucao@gmail.com',
      url='https://github.com/twocucao/doumu.fm',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=True,
      install_requires=[
          'requests',
      ],
      entry_points={
          'console_scripts': [
              'danmu.fm = danmufm.danmu:main'
          ]
      },
      )

# install_requires=[
# 'requests',
# 'pycookiecheat'
# ] + (['pyobjc-core', 'pyobjc'] if 'darwin' in sys.platform else []),



#!/usr/bin/env python
# encoding: utf-8

#!/usr/bin/env python
# encoding: utf-8

'''
静态配置
'''
import logging
from logging.config import dictConfig
import sys
import os
import shutil
from os.path import expanduser

VERSION               = "0.3.0"
PROJECT_NAME          = "danmufm"
MODULE_NAME           = PROJECT_NAME.lower()
# 项目根目录
ROOT_PROJECT_DIR      = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
# 项目次目录
MODULE_DIR            = os.path.join(ROOT_PROJECT_DIR, MODULE_NAME)
TEST_DIR              = os.path.join(ROOT_PROJECT_DIR, 'test')
TEMPLATE_DIR          = os.path.join(MODULE_DIR, 'template')

CURRENT_USER_HOME_DIR = expanduser("~")
USER_CONFIG_DIR       = os.path.join(CURRENT_USER_HOME_DIR, '.danmu.fm')
REPORTS_DIR           = os.path.join(USER_CONFIG_DIR, 'reports')
LOGS_DIR              = os.path.join(USER_CONFIG_DIR, 'logs')

if not os.path.exists(USER_CONFIG_DIR):
    os.makedirs(USER_CONFIG_DIR)

if not os.path.exists(REPORTS_DIR):
    os.makedirs(REPORTS_DIR)

if not os.path.exists(LOGS_DIR):
    os.makedirs(LOGS_DIR)

# 拷贝模板配置文件
CONFIG_FILE = os.path.join(USER_CONFIG_DIR,"danmu.fm.conf")
if not os.path.exists(CONFIG_FILE):
    shutil.copy(os.path.join(TEMPLATE_DIR,"danmu.fm.conf"),USER_CONFIG_DIR)

config = {}

with open(CONFIG_FILE) as f:
    code = compile(f.read(), CONFIG_FILE, 'exec')
    exec(code, config)
# 数据库配置

# 日志配置
## 开启DEBUG以上
DEBUG = config["DEBUG"] # Whether or not to show DEBUG level messages
## 开启日志颜色
USE_COLORS = config["USE_COLORS"] # Whether or not colors should be used when outputting text

# danmu.client
# danmu.fm 文件即可
LOGGING = {  # dictConfig for output stream and file logging
    'version': 1,
    'disable_existing_loggers': False,

    'formatters': {
        'console': {
            'format': '[%(asctime)s] %(levelname)s::%(module)s - %(message)s',
        },
        'file': {
            'format': '[%(asctime)s] %(levelname)s::(P:%(process)d T:%(thread)d)::%(module)s - %(message)s',
        },
    },

    'handlers': {
        'console': {
            'class': MODULE_NAME + '.misc.color_stream_handler.ColorStreamHandler',
            'formatter': 'console',
            'level': 'DEBUG',
            'use_colors': USE_COLORS,
        },
        'file': {
            'class': 'logging.handlers.TimedRotatingFileHandler',
            'formatter': 'file',
            'level': 'INFO',
            'when': 'midnight',
            'filename': str(LOGS_DIR) + '/danmu.fm.log',
            'interval': 1,
            'backupCount': 0,
            'encoding': None,
            'delay': False,
            'utc': False,
        },
    },

    'loggers': {
        'default': {
            'handlers': ['console'],
            'level': 'DEBUG' if DEBUG else 'INFO',
            'propagate': True,
        },
        'danmu.client': {
            'handlers': ['console'],
            'level': 'DEBUG' if DEBUG else 'INFO',
            'propagate': True,
        },
        'danmu.fm': {
            'handlers': ['console', 'file'],
            'level': 'DEBUG' if DEBUG else 'INFO',
            'propagate': True,
        }
    }
}
dictConfig(LOGGING)

## 邮件设置

MAIL_SMTP_HOST = config["MAIL_SMTP_HOST"]
MAIL_USER      = config["MAIL_USER"]
MAIL_PASSWORD  = config["MAIL_PASSWORD"]



# 检查配置使用,真实环境中应该取消
# print(locals())





# [TODO: 添加设置报告]
